// Copyright (c) 2016, Tobias Mueller tm(at)tm3d.de
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the
//    distribution.
//  * All advertising materials mentioning features or use of this
//    software must display the following acknowledgement: This product
//    includes software developed by tm3d.de and its contributors.
//  * Neither the name of tm3d.de nor the names of its contributors may
//    be used to endorse or promote products derived from this software
//    without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include "hexfile.h"

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <sstream>

hexFile::hexFile(std::string fname) {
	std::ifstream hf(fname);
	if (!hf) {
		lastError=3;
		return;
	}
	std::string line;
	std::stringstream ss;
	lastError=0;
	while (std::getline(hf, line))    {
        //QueryArray.push_back(line);
        if (line[0]==':') {
			//ss << std::hex << line.substr(1,2);
			//ss >> bc;
			uint8_t bc=std::stoul( line.substr(1,2), nullptr, 16);
			uint32_t fw=std::stoul(line.substr(3,4),nullptr, 16);
			uint8_t ty=std::stoul(line.substr(7,2),nullptr,16);
			uint32_t chsm = bc + (fw >> 8) + (fw & 0xFF) + ty;
			for (int i = 0; i < bc; i++) {
				int p = 9 + i * 2;
				uint8_t b = std::stoul(line.substr(p, 2),nullptr, 16);
				chsm = chsm + b;
				prog.push_back(b);
			}
			chsm=(chsm+std::stoul(line.substr(9+bc*2,2),nullptr,16))&0xFF;
			if (chsm!=0) {
				lastError=1;
				return;
				//std::cout <<"ERROR"<<std::endl;
			}
				
			//std::cout<<static_cast<int>(bc)<<" "<< line<<std::endl;
		}
    }
    if (prog.size() < 1){
		lastError = 2;
	}
	
}

std::vector<uint8_t> hexFile::getBlock(unsigned int pos,unsigned int length){
	std::vector<uint8_t> rvec1;
	if (pos>prog.size()) return rvec1;
	if ((pos+length) > prog.size()) {
		std::vector<uint8_t> rvec(prog.begin()+pos,prog.end());
		return rvec;
	}else {
		std::vector<uint8_t> rvec(prog.begin()+pos,prog.begin()+pos+length);
		return rvec;
	}
}

unsigned int hexFile::getBlockCount(unsigned int length){
	unsigned int count=prog.size()/length;
	if ((prog.size()%length)!=0) count++;
	return count;
}

